import { HashMap, Map } from "../util/map";
import { IllegalArgumentException } from "./exception";

export class Enum<SubEnum> {
  private static enums: Map<Function, any[]> = new HashMap();

  protected constructor() {
    let enums = Enum.enums,
      constructor = Object.getPrototypeOf( this ).constructor;

    if ( !enums.containsKey( constructor ) ) {
      enums.put( constructor, [] );
    }

    enums.get( constructor ).push( this );
  }

  // 方法名定义为 'valueOf' 时再使用 this 指针会出现隐藏的循环调用
  public static of<SubEnum>( name: string ): SubEnum {
    if ( !this.hasOwnProperty( name ) ) {
      throw new IllegalArgumentException( `Enum '${name}' is not found from ${this.name}` );
    }

    return ( <any>this )[ name ];
  }

  public static values<SubEnum>(): SubEnum[] {
    return Enum.enums.getOrDefault( this, [] );
  }
}